@@ -237,7 +237,7 @@ bool PreprocessSPVIR::runOnModule(Module &M) {
   // signatures generated by Clang, given that it only has partial TargetExtTy
   // support.
 #if LLVM_VERSION_MAJOR >= 16
-  retypeOpenCLTargetExtTyArgs(&M);
+  retypeOpenCLTargetExtTyAsPointers(&M);
 #endif
 
   return m_changed;
