@@ -845,7 +845,8 @@ void ConstantCoalescing::MergeUniformLoad(Instruction *load, Value *bufIdxV, uin
   auto shouldMerge = [&](const BufChunk *cur_chunk) {
     if (CompareBufferBase(cur_chunk->bufIdxV, cur_chunk->addrSpace, bufIdxV, addrSpace) &&
         cur_chunk->baseIdxV == eltIdxV && cur_chunk->chunkIO->getType()->getScalarType() == loadEltTy &&
-        CompareMetadata(cur_chunk->chunkIO, load) && !CheckForAliasingWrites(addrSpace, cur_chunk->chunkIO, load)) {
+        CompareMetadata(cur_chunk->chunkIO, load) && !CheckForAliasingWrites(addrSpace, cur_chunk->chunkIO, load) &&
+        cur_chunk->extensionKind == Extension) {
       uint lb = std::min(eltid, cur_chunk->chunkStart);
       uint ub = std::max(eltid + maxEltPlus, cur_chunk->chunkStart + cur_chunk->chunkSize);
       if (profitableChunkSize(ub, lb, scalarSizeInBytes) && (isDwordAligned || eltid >= cur_chunk->chunkStart)) {
@@ -882,8 +883,6 @@ void ConstantCoalescing::MergeUniformLoad(Instruction *load, Value *bufIdxV, uin
   const uint32_t loadNumElements =
       loadDataTy->isVectorTy() ? int_cast<uint32_t>(cast<IGCLLVM::FixedVectorType>(loadDataTy)->getNumElements()) : 1;
   // VectorPreProcess pass legalizes loaded data size.
-  IGC_ASSERT(loadNumElements == RoundChunkSize(loadNumElements));
-  IGC_ASSERT(loadNumElements >= RoundChunkSize(maxEltPlus));
   if (!cov_chunk) {
     if (isDwordAligned) {
       cov_chunk = new BufChunk();
@@ -893,6 +892,7 @@ void ConstantCoalescing::MergeUniformLoad(Instruction *load, Value *bufIdxV, uin
       cov_chunk->elementSize = scalarSizeInBytes;
       cov_chunk->chunkStart = eltid;
       cov_chunk->chunkSize = RoundChunkSize(maxEltPlus);
+      cov_chunk->extensionKind = Extension;
       const alignment_t chunkAlignment = std::max<alignment_t>(alignment, 4);
       cov_chunk->chunkIO = CreateChunkLoad(load, cov_chunk, eltid, chunkAlignment, Extension);
       chunk_vec.push_back(cov_chunk);
@@ -1767,7 +1767,6 @@ void ConstantCoalescing::ScatterToSampler(Instruction *load, Value *bufIdxV, uin
   IGC_ASSERT((!load->getType()->isVectorTy()) ||
              (cast<IGCLLVM::FixedVectorType>(load->getType())->getNumElements() <= 4));
 
-  const bool useByteAddress = m_ctx->m_DriverInfo.UsesTypedConstantBuffersWithByteAddress();
 
   // Code below doesn't support crossing 4 DWORD boundary i.e. mapping a
   // single input load to multiple sampler loads.
@@ -1780,33 +1779,28 @@ void ConstantCoalescing::ScatterToSampler(Instruction *load, Value *bufIdxV, uin
     irBuilder->SetInsertPoint(baseInBytes);
 
     WIAnalysis::WIDependancy baseInBytesDep = wiAns->whichDepend(baseInBytes);
+    Value *baseAddressInOwords = GetSamplerAlignedAddress(baseInBytes);
+    IGC_ASSERT(baseAddressInOwords);
 
-    // Data address for sampler load, either in OWORDs or in bytes
-    Value *chunkBaseAddress = baseInBytes;
-    if (!useByteAddress) {
-      // base address is in OWORDs
-      Value *baseAddressInOwords = GetSamplerAlignedAddress(baseInBytes);
-      IGC_ASSERT(baseAddressInOwords);
-
-      // it is possible that baseInBytes is uniform, yet load is non-uniform due to the use location of load
-      if (baseAddressInOwords != baseInBytes->getOperand(0)) {
-        Value *newVal = irBuilder->CreateShl(baseAddressInOwords, ConstantInt::get(baseAddressInOwords->getType(), 4));
-        wiAns->incUpdateDepend(newVal, baseInBytesDep);
-        baseInBytes->replaceAllUsesWith(newVal);
-      } else if (wiAns->whichDepend(baseAddressInOwords) != baseInBytesDep) {
-        // quick fix for a special case: baseAddressInOwords is uniform and baseInBytes is not uniform.
-        // If we use baseInBytes-src0 (elementIndx) directly at cf-join point by this transform,
-        // we can change the uniformness of baseAddressInOwords
-        baseAddressInOwords =
-            irBuilder->CreateShl(baseAddressInOwords, ConstantInt::get(baseAddressInOwords->getType(), 0));
-        wiAns->incUpdateDepend(baseAddressInOwords, baseInBytesDep);
-        Value *newVal = irBuilder->CreateShl(baseAddressInOwords, ConstantInt::get(baseAddressInOwords->getType(), 4));
-        wiAns->incUpdateDepend(newVal, baseInBytesDep);
-        baseInBytes->replaceAllUsesWith(newVal);
-      }
-
-      chunkBaseAddress = baseAddressInOwords;
+    // it is possible that baseInBytes is uniform, yet load is non-uniform due to the use location of load
+    if (baseAddressInOwords != baseInBytes->getOperand(0)) {
+      Value *newVal = irBuilder->CreateShl(baseAddressInOwords, ConstantInt::get(baseAddressInOwords->getType(), 4));
+      wiAns->incUpdateDepend(newVal, baseInBytesDep);
+      baseInBytes->replaceAllUsesWith(newVal);
+    } else if (wiAns->whichDepend(baseAddressInOwords) != baseInBytesDep) {
+      // quick fix for a special case: baseAddressInOwords is uniform and baseInBytes is not uniform.
+      // If we use baseInBytes-src0 (elementIndx) directly at cf-join point by this transform,
+      // we can change the uniformness of baseAddressInOwords
+      baseAddressInOwords =
+          irBuilder->CreateShl(baseAddressInOwords, ConstantInt::get(baseAddressInOwords->getType(), 0));
+      wiAns->incUpdateDepend(baseAddressInOwords, baseInBytesDep);
+      Value *newVal = irBuilder->CreateShl(baseAddressInOwords, ConstantInt::get(baseAddressInOwords->getType(), 4));
+      wiAns->incUpdateDepend(newVal, baseInBytesDep);
+      baseInBytes->replaceAllUsesWith(newVal);
     }
+
+    Value *chunkBaseAddress = baseAddressInOwords;
+
     BufChunk *cov_chunk = nullptr;
     for (std::vector<BufChunk *>::reverse_iterator rit = chunk_vec.rbegin(), rie = chunk_vec.rend(); rit != rie;
          ++rit) {
@@ -1836,12 +1830,11 @@ void ConstantCoalescing::ScatterToSampler(Instruction *load, Value *bufIdxV, uin
 
       Value *dataAddress = chunkBaseAddress;
       if (offsetInBytes >= samplerLoadSizeInBytes) {
-        const uint32_t chunkOffset = (useByteAddress) ? (cov_chunk->chunkStart * cov_chunk->elementSize) : // in bytes
-                                         (offsetInBytes / samplerLoadSizeInBytes);                         // in OWORDs
+        const uint32_t chunkOffset = offsetInBytes / samplerLoadSizeInBytes;
         dataAddress = irBuilder->CreateAdd(dataAddress, ConstantInt::get(dataAddress->getType(), chunkOffset));
         wiAns->incUpdateDepend(dataAddress, WIAnalysis::RANDOM);
       }
-      if (dataAddress->getType()->getIntegerBitWidth() >= 32 && !useByteAddress) {
+      if (dataAddress->getType()->getIntegerBitWidth() >= 32) {
         dataAddress = irBuilder->CreateAnd(dataAddress, ConstantInt::get(dataAddress->getType(), 0x0FFFFFFF));
         wiAns->incUpdateDepend(dataAddress, WIAnalysis::RANDOM);
       }
