@@ -84,12 +84,18 @@ public:
 
   void CreateKernelShaderMap(CodeGenContext *ctx, IGC::IGCMD::MetaDataUtils *pMdUtils, llvm::Function &F);
 
+
+  bool isVectorEmissionPossible(const SSource sources[2], CVariable *src[2]);
+
   void Frc(const SSource &source, const DstModifier &modifier);
   void Floor(const SSource &source, const DstModifier &modifier);
   void Mad(const SSource sources[3], const DstModifier &modifier);
   void Lrp(const SSource sources[3], const DstModifier &modifier);
   void Cmp(llvm::CmpInst::Predicate pred, const SSource sources[2], const DstModifier &modifier,
            uint8_t clearTagMask = 0);
+  void VectorCMP(llvm::CmpInst::Predicate pred, const SSource sources[2], const DstModifier &modifier,
+           uint8_t clearTagMask = 0);
+
   void Sub(const SSource[2], const DstModifier &mofidier);
   void Xor(const SSource[2], const DstModifier &modifier);
   void FDiv(const SSource[2], const DstModifier &modifier);
@@ -97,6 +103,8 @@ public:
   void Pow(const SSource sources[2], const DstModifier &modifier);
   void Avg(const SSource sources[2], const DstModifier &modifier);
   void Rsqrt(const SSource &source, const DstModifier &modifier);
+
+  void VectorSelect(const SSource sources[3], const DstModifier &modifier);
   void Select(const SSource sources[3], const DstModifier &modifier);
   void PredAdd(const SSource &pred, bool invert, const SSource sources[2], const DstModifier &modifier);
   void Mul(const SSource[2], const DstModifier &modifier);
