@@ -9,6 +9,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
 #include "AdaptorOCL/MergeAllocasOCL.h"
 #include "Compiler/Legalizer/PeepholeTypeLegalizer.hpp"
+#include "Compiler/CISACodeGen/DropTargetBBs.hpp"
 #include "Compiler/CISACodeGen/layout.hpp"
 #include "Compiler/CISACodeGen/DeSSA.hpp"
 #include "Compiler/CISACodeGen/GenCodeGenModule.h"
@@ -78,6 +79,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/Optimizer/OpenCLPasses/MergeScalarPhisPass/MergeScalarPhisPass.hpp"
 #include "Compiler/Legalizer/AddRequiredMemoryFences.h"
 #include "Compiler/Optimizer/OpenCLPasses/GenericAddressResolution/GenericAddressDynamicResolution.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/GenericAddressResolution/GenericNullPtrPropagation.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/PrivateMemory/PrivateMemoryToSLM.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ProgramScopeConstants/ProgramScopeConstantResolution.hpp"
@@ -175,6 +177,8 @@ SPDX-License-Identifier: MIT
 #include "Compiler/TranslateToProgrammableOffsetsPass.hpp"
 #include "Compiler/CISACodeGen/RemoveLoopDependency.hpp"
 
+#include <filesystem>
+
 /***********************************************************************************
 This file contains the generic code generation functions for all the shaders
 The class CShader is inherited for each specific type of shaders to add specific
@@ -364,8 +368,17 @@ void AddAnalysisPasses(CodeGenContext &ctx, IGCPassManager &mpm) {
   mpm.add(createCleanPHINodePass());
   if (IGC_IS_FLAG_SET(DumpRegPressureEstimate))
     mpm.add(new IGCRegisterPressurePrinter("final"));
+
+  // save RPE results in metadata
+  if (ctx.type == ShaderType::OPENCL_SHADER) {
+    mpm.add(new IGCRegisterPressurePublisher());
+  }
+
   // Let Layout be the last pass before Emit Pass
   mpm.add(new Layout());
+  if (IGC_IS_FLAG_ENABLED(EnableDropTargetBBs)) {
+    mpm.add(new DropTargetBBs());
+  }
 
 
   mpm.add(createTimeStatsCounterPass(&ctx, TIME_CG_Analysis, STATS_COUNTER_END));
@@ -565,6 +578,10 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
     mpm.add(new PromoteResourceToDirectAS());
   }
 
+  if (!isOptDisabled) {
+    mpm.add(createPruneUnusedArgumentsPass());
+  }
+
   if (ctx.m_instrTypes.hasReadOnlyArray) {
     mpm.add(createDeadCodeEliminationPass());
     mpm.add(createSROAPass());
@@ -577,6 +594,8 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
       mpm.add(createResolveGASPass());
     }
     mpm.add(createGenericAddressDynamicResolutionPass());
+    mpm.add(createDeadCodeEliminationPass());
+    mpm.add(createGenericNullPtrPropagationPass());
   }
 
   // Resolve the Private memory to register pass
@@ -852,6 +871,34 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
     }
 
     mpm.add(createCloneAddressArithmeticPass());
+    // cloneAddressArithmetic leaves old instructions unnecessary
+    // dce pass helps to clean that up
+    mpm.add(createDeadCodeEliminationPass());
+    if (IGC_IS_FLAG_SET(DumpRegPressureEstimate))
+      mpm.add(new IGCRegisterPressurePrinter("after_remat"));
+  } else if (ctx.m_retryManager.AllowCloneAddressArithmetic() && IGC_GET_FLAG_VALUE(RematOptionsForRetry) ||
+             ctx.platform.supportsVRT() && IGC_GET_FLAG_VALUE(RematOptionsForVRT)) {
+
+    if (IGC_GET_FLAG_VALUE(RematInstCombineBefore))
+      mpm.add(createIGCInstructionCombiningPass());
+
+    // see comment above
+    if (IGC_GET_FLAG_VALUE(RematReassocBefore)) {
+      mpm.add(llvm::createReassociatePass());
+      mpm.add(llvm::createEarlyCSEPass());
+      mpm.add(llvm::createReassociatePass());
+      mpm.add(llvm::createEarlyCSEPass());
+      mpm.add(llvm::createReassociatePass());
+      mpm.add(llvm::createEarlyCSEPass());
+    }
+
+    // if both retry and VRT checks go through, retry is more important
+    auto rematOptions = ctx.m_retryManager.AllowCloneAddressArithmetic() && IGC_GET_FLAG_VALUE(RematOptionsForRetry)
+                            ? static_cast<IGC::REMAT_OPTIONS>(IGC_GET_FLAG_VALUE(RematOptionsForRetry))
+                            : static_cast<IGC::REMAT_OPTIONS>(IGC_GET_FLAG_VALUE(RematOptionsForVRT));
+
+    mpm.add(createCloneAddressArithmeticPassWithFlags(rematOptions));
+
     // cloneAddressArithmetic leaves old instructions unnecessary
     // dce pass helps to clean that up
     mpm.add(createDeadCodeEliminationPass());
