@@ -64,6 +64,11 @@ bool RetryManager::AdvanceState() {
   return (stateId < RetryTableSize);
 }
 
+void RetryManager::DecreaseState() {
+  IGC_ASSERT(stateId < RetryTableSize);
+  stateId = prevStateId;
+}
+
 unsigned RetryManager::GetPerFuncRetryStateId(Function *F) const {
   if (IGC_GET_FLAG_VALUE(AllowStackCallRetry) == 2 && F != nullptr && prevStateId < RetryTableSize &&
       !PerFuncRetrySet.empty()) {
@@ -321,7 +326,6 @@ LLVMContextWrapper::LLVMContextWrapper(bool createResourceDimTypes) : m_UserAddr
   if (WA_OpaquePointersCL && WA_OpaquePointersCL->getNumOccurrences() > 0) {
     IGC_IsPointerModeAlreadySet = true;
   }
-  
   if (IGC::canOverwriteLLVMCtxPtrMode(basePtr, IGC_IsPointerModeAlreadySet)) {
     bool enableOpaquePointers = AreOpaquePointersEnabled();
     IGCLLVM::setOpaquePointers(basePtr, enableOpaquePointers);
@@ -389,6 +393,9 @@ void CodeGenContext::print(llvm::raw_ostream &stream) const {
 }
 
 void CodeGenContext::initLLVMContextWrapper(bool createResourceDimTypes) {
+  if (llvmCtxWrapper)
+    llvmCtxWrapper->Release();
+
   llvmCtxWrapper = new LLVMContextWrapper(createResourceDimTypes);
   llvmCtxWrapper->AddRef();
 }
@@ -704,12 +711,43 @@ void CodeGenContext::EmitWarning(const char *warningstr, const llvm::Value *cont
 
 CompOptions &CodeGenContext::getCompilerOption() { return getModuleMetaData()->compOpt; }
 
-void CodeGenContext::resetOnRetry() { m_tempCount = 0; }
+void CodeGenContext::resetOnRetry(bool isSubmodule) {
+
+  m_tempCount = 0;
+
+  if (isSubmodule)
+    return;
+
+  clear();
+  initLLVMContextWrapper();
+}
 
 int32_t CodeGenContext::getNumThreadsPerEU() const { return -1; }
 
 uint32_t CodeGenContext::getExpGRFSize() const { return 0; }
 
+bool CodeGenContext::allowATOB()
+{
+  bool allow = type == ShaderType::COMPUTE_SHADER ||
+    (type == ShaderType::OPENCL_SHADER &&
+      !(getModuleMetaData()->NBarrierCnt > 0));
+
+  if (m_instrTypes.hasSplitBarrier &&
+    m_instrTypes.hasWorkgroupBarrier)
+  {
+    // The regular and split barrier cannnot share the
+    // same ID, because we could have such scenario:
+    // splitbarrier.signal();
+    // workgroupbarrier();
+    // splitbarrier.wait();
+    // and this will cause a hang.
+    // The split barrier will use the ID 1
+    // so, we cannot setup ATOB for this case
+    allow = false;
+  }
+  return allow;
+}
+
 /// parameter "returnDefault" controls what to return when
 /// there is no user-forced setting
 uint32_t CodeGenContext::getNumGRFPerThread(bool returnDefault) {
