@@ -71,6 +71,7 @@ void initializeLowerGPCallArgPass(llvm::PassRegistry &);
 void initializeGenerateBlockMemOpsPassPass(llvm::PassRegistry &);
 void initializeGenericAddressAnalysisPass(llvm::PassRegistry &);
 void initializeGenericAddressDynamicResolutionPass(llvm::PassRegistry &);
+void initializeGenericNullPtrPropagationPass(llvm::PassRegistry &);
 void initializeGenFDIVEmulationPass(llvm::PassRegistry &);
 void initializeGenIRLoweringPass(llvm::PassRegistry &);
 void initializeGEPLoopStrengthReductionPass(llvm::PassRegistry &);
@@ -101,6 +102,7 @@ void initializeInlineLocalsResolutionPass(llvm::PassRegistry &);
 void initializeInlineUnmaskedFunctionsPassPass(llvm::PassRegistry &);
 void initializeInsertBranchOptPass(llvm::PassRegistry &);
 void initializeHFOptPass(llvm::PassRegistry &);
+void initializeDropTargetBBsPass(llvm::PassRegistry &);
 void initializeLayoutPass(llvm::PassRegistry &);
 void initializeLdShrinkPass(llvm::PassRegistry &);
 void initializeLegalizationPass(llvm::PassRegistry &);
@@ -164,6 +166,8 @@ void initializeStatelessToStatefulPass(llvm::PassRegistry &);
 void initializeSubGroupFuncsResolutionPass(llvm::PassRegistry &);
 void initializeSubGroupReductionPatternPass(llvm::PassRegistry &);
 void initializeTransformUnmaskedFunctionsPassPass(llvm::PassRegistry &);
+void initializeDisableInliningPass(llvm::PassRegistry &);
+void initializeDropTargetFunctionsPass(llvm::PassRegistry &);
 void initializeIndirectCallOptimizationPass(llvm::PassRegistry &);
 void initializePromoteInt8TypePass(llvm::PassRegistry &);
 void initializeDpasFuncsResolutionPass(llvm::PassRegistry &);
@@ -267,12 +271,14 @@ void initializeRematChainsAnalysisPass(llvm::PassRegistry &);
 void initializeVectorShuffleAnalysisPass(llvm::PassRegistry &);
 void initializeIGCLivenessAnalysisPass(llvm::PassRegistry &);
 void initializeIGCRegisterPressurePrinterPass(llvm::PassRegistry &);
+void initializeIGCRegisterPressurePublisherPass(llvm::PassRegistry &);
 void initializeIGCVectorizerPass(llvm::PassRegistry &);
 void initializeIGCFunctionExternalRegPressureAnalysisPass(llvm::PassRegistry &);
 void initializePromoteConstantStructsPass(llvm::PassRegistry &);
 void initializeLowerInvokeSIMDPass(llvm::PassRegistry &);
 void initializeRemoveCodeAssumptionsPass(llvm::PassRegistry &);
 void initializePreprocessSPVIRPass(llvm::PassRegistry &);
+void initializeConvertUserSemanticDecoratorOnFunctionsPass(llvm::PassRegistry &);
 void initializePromoteBoolsPass(llvm::PassRegistry &);
 void initializeResolveConstExprCallsPass(llvm::PassRegistry &);
 void initializeHandleSpirvDecorationMetadataPass(llvm::PassRegistry &);
@@ -294,3 +300,4 @@ void initializeInjectPrintfPass(llvm::PassRegistry &);
 void initializeWaveShuffleIndexSinkingPass(llvm::PassRegistry &);
 void initializeInstructionHoistingOptimizationPass(llvm::PassRegistry &);
 void initializeWaveBallotCSEPass(llvm::PassRegistry &);
+void initializeProcessBICodeAssumptionPass(llvm::PassRegistry &);
