@@ -41,13 +41,7 @@ llvm::Expected<object::ELF64LEFile> getElfFile(llvm::StringRef ZeBinary) {
   auto ElfOrErr = object::ObjectFile::createELFObjectFile(inputRef);
   if (!ElfOrErr)
     return ElfOrErr.takeError();
-#if LLVM_VERSION_MAJOR < 12
-  auto ElfFilePointer = cast<object::ELF64LEObjectFile>(*ElfOrErr.get()).getELFFile();
-  IGC_ASSERT(ElfFilePointer);
-  auto ElfFile = *ElfFilePointer;
-#else
   auto ElfFile = cast<object::ELF64LEObjectFile>(*ElfOrErr.get()).getELFFile();
-#endif
 
   return ElfFile;
 }
@@ -60,7 +54,7 @@ llvm::Expected<std::vector<llvm::StringRef>> getZeBinSectionsData(llvm::StringRe
   auto ElfFileOrErr = getElfFile(ZeBinary);
   if (!ElfFileOrErr)
     return ElfFileOrErr.takeError();
-  auto ElfFile = ElfFileOrErr.get();
+  const auto &ElfFile = ElfFileOrErr.get();
 
   auto ElfSections = ElfFile.sections();
   if (!ElfSections)
@@ -68,11 +62,7 @@ llvm::Expected<std::vector<llvm::StringRef>> getZeBinSectionsData(llvm::StringRe
 
   for (auto &Sect : *ElfSections) {
     if (Sect.sh_type == SectionType) {
-#if LLVM_VERSION_MAJOR < 12
-      auto SectionDataOrErr = ElfFile.getSectionContents(&Sect);
-#else
       auto SectionDataOrErr = ElfFile.getSectionContents(Sect);
-#endif
       if (!SectionDataOrErr)
         return SectionDataOrErr.takeError();
       StringRef Data(reinterpret_cast<const char *>((*SectionDataOrErr).data()), (size_t)Sect.sh_size);
@@ -451,11 +441,7 @@ bool TranslateBuildSPMDAndESIMD(llvm::ArrayRef<SPVTranslationPair> InputModules,
 
     auto &InputArgs = InputArgsPair.second;
 
-    TC::STB_TranslateOutputArgs NewOutputArgs;
-    CIF::SafeZeroOut(NewOutputArgs);
-    auto outputData = std::unique_ptr<char[]>(NewOutputArgs.pOutput);
-    auto errorString = std::unique_ptr<char[]>(NewOutputArgs.pErrorString);
-    auto debugData = std::unique_ptr<char[]>(NewOutputArgs.pDebugData);
+    TC::STB_TranslateOutputArgs NewOutputArgs{};
 
     STB_TranslateInputArgs NewInputArgs = InputArgs;
     std::string NewInternalOptions{InputArgs.pInternalOptions ? InputArgs.pInternalOptions : ""};
@@ -514,9 +500,9 @@ bool TranslateBuildSPMDAndESIMD(llvm::ArrayRef<SPVTranslationPair> InputModules,
     }
 
     if (!success) {
-      if (errorMessage.empty() && NewOutputArgs.pErrorString) {
+      if (errorMessage.empty() && !NewOutputArgs.ErrorString.empty()) {
         errorMessage = "VLD: Failed to compile SPIR-V with following error: \n";
-        errorMessage += NewOutputArgs.pErrorString;
+        errorMessage += NewOutputArgs.ErrorString;
       }
       return false;
     }
@@ -524,11 +510,11 @@ bool TranslateBuildSPMDAndESIMD(llvm::ArrayRef<SPVTranslationPair> InputModules,
     // If this is the last SPIR-V to compile, stop here. The rest of the code
     // handles extracting information for further compilations.
     if (IsLast) {
-      *pOutputArgs = NewOutputArgs;
+      *pOutputArgs = std::move(NewOutputArgs);
       break;
     }
 
-    llvm::StringRef ZeBinary(NewOutputArgs.pOutput, NewOutputArgs.OutputSize);
+    llvm::StringRef ZeBinary(NewOutputArgs.Output.data(), NewOutputArgs.Output.size());
 
     // Check if the output has ZEINFO section. If not, it can mean that the
     // module didn't contain any exported functions.
