@@ -9,7 +9,9 @@ SPDX-License-Identifier: MIT
 #ifndef IGCLLVM_ADT_OPTIONAL_H
 #define IGCLLVM_ADT_OPTIONAL_H
 
+#if LLVM_VERSION_MAJOR < 17
 #include <llvm/ADT/Optional.h>
+#endif
 #include <optional>
 
 namespace IGCLLVM {
@@ -23,23 +25,21 @@ template <class T> using optional = std::optional<T>;
 template <class T> T getValue(optional<T> data) { return data.value(); }
 #endif
 
-template <typename T>
-    static std::optional<T> makeOptional(const llvm::Optional<T>& O) {
+template <typename T> static std::optional<T> makeOptional(const llvm::Optional<T> &O) {
 #if LLVM_VERSION_MAJOR < 16
-        return O.hasValue() ? std::optional<T>(O.getValue()) : std::optional<T>(std::nullopt);
+  return O.hasValue() ? std::optional<T>(O.getValue()) : std::optional<T>(std::nullopt);
 #else
-        return O.has_value() ? std::optional<T>(O.value()) : std::optional<T>(std::nullopt);
+  return O.has_value() ? std::optional<T>(O.value()) : std::optional<T>(std::nullopt);
 #endif
-    }
+}
 
-    template <typename T>
-    optional<T> makeLLVMOptional(const std::optional<T> &O) {
+template <typename T> optional<T> makeLLVMOptional(const std::optional<T> &O) {
 #if LLVM_VERSION_MAJOR < 16
-      return O.has_value() ? llvm::Optional<T>(O.value()) : llvm::Optional<T>();
+  return O.has_value() ? llvm::Optional<T>(O.value()) : llvm::Optional<T>();
 #else
-      return O;
+  return O;
 #endif
-    }
+}
 } // namespace IGCLLVM
 
 #endif // IGCLLVM_ADT_OPTIONAL_H
