@@ -36,14 +36,14 @@ namespace llvm
     class Type;
     }
 
-const unsigned int INPUT_RESOURCE_SLOT_COUNT = 128;
-const unsigned int NUM_SHADER_RESOURCE_VIEW_SIZE = (INPUT_RESOURCE_SLOT_COUNT + 1) / 64;
-
 const unsigned int g_c_maxNumberOfBufferPushed = 4;
 static const int MAX_VECTOR_SIZE_TO_PRINT_IN_SHADER_DUMPS = 1000;
 
 namespace IGC
 {
+    const unsigned int INPUT_RESOURCE_SLOT_COUNT = 128;
+    const unsigned int NUM_SHADER_RESOURCE_VIEW_SIZE = (INPUT_RESOURCE_SLOT_COUNT + 1) / 64;
+
     const unsigned int INVALID_CONSTANT_BUFFER_INVALID_ADDR = 0xFFFFFFFF;
 
     enum FunctionTypeMD
@@ -366,6 +366,8 @@ enum class ShaderTypeMD
         // This is true if the function has any sync raytracing functionality
         bool hasSyncRTCalls = false;
         bool hasPrintfCalls = false;
+        bool requireAssertBuffer = false;
+        bool requireSyncBuffer = false;
         bool hasIndirectCalls = false;
 
         // Analysis result of if there are non-kernel-argument ld/st in the kernel
@@ -502,7 +504,7 @@ enum class ShaderTypeMD
         unsigned FastestS1Options                       = 0;  // FCEXP_NO_EXPRIMENT. Can't access the enum here for some reason.
         bool DisableFastestForWaveIntrinsicsCS          = false;
         bool ForceLinearWalkOnLinearUAV                 = false;
-        bool DisableLscSamplerRouting                   = false;
+        unsigned LscSamplerRouting                      = 0;
         bool UseBarrierControlFlowOptimization          = false;
         bool EnableDynamicRQManagement                  = false;
         bool WaDisablePayloadCoalescing                 = false;
@@ -513,6 +515,7 @@ enum class ShaderTypeMD
         bool UseInstructionHoistingOptimization         = false;
         bool DisableResourceLoopDestLifeTimeStart       = false;
         bool UseLinearScanRA                            = false;
+        bool DisableConvertingAtomicIAddToIncDec        = false;
     };
 
     enum class ThreadIDLayout
@@ -534,6 +537,7 @@ enum class ShaderTypeMD
         unsigned int forceTotalGRFNum = 0; // 0 means not forced
         unsigned int VISAPreSchedRPThreshold = 0; // 0 means use the default
         unsigned int VISAPreSchedCtrl = 0; // 0 means use the default
+        unsigned int VISAPreSchedScheduleExtraGRF = 0; // additional GRF for latency scheduling
         unsigned int SetLoopUnrollThreshold = 0; // 0 means use the default
         bool forceSpillCompression = false;
         bool allowLowerSimd = false;
