@@ -237,8 +237,6 @@ void DbgDecoder::ddCalleeCallerSave(uint32_t relocOffset, CallerCallee callCase)
 }
 
 int DbgDecoder::ddDbg() {
-  dbgFile = fopen(filename, "rb");
-
   if (!dbgFile) {
     std::cerr << "Error opening and creating debug file: " << filename << "\n";
     vISA_ASSERT(false, "Unable to write debug file to disk.");
@@ -260,8 +258,6 @@ int DbgDecoder::ddDbg() {
               << " *************"
               << "\n";
 
-    fclose(dbgFile);
-
     return -1;
   }
 
@@ -453,8 +449,6 @@ int DbgDecoder::ddDbg() {
 
   std::cout << "=== End of Debug Dump ===\n";
 
-  fclose(dbgFile);
-
   return 0;
 }
 
@@ -2050,8 +2044,7 @@ INST_LIST KernelDebugInfo::getDeltaInstructions(G4_BB *bb) {
 
 void SaveRestoreManager::addInst(G4_INST *inst) {
   const IR_Builder &builder = inst->getBuilder();
-  SaveRestoreInfo newSVInfo = {};
-  srInfo.push_back(newSVInfo);
+  srInfo.emplace_back();
   if (srInfo.size() > 1) {
     // Copy over from previous
     // so emitted data is
@@ -2241,7 +2234,7 @@ void SaveRestoreInfo::update(G4_INST *inst, int32_t memOffset,
                                 << m.offset << " bytes" << offstr << "\n");
       }
     } else if (inst->getMsgDesc()->isRead()) {
-      uint32_t srcreg, dstreg;
+      [[maybe_unused]] uint32_t srcreg, dstreg;
       srcreg = inst->getSrc(0)->getLinearizedStart() /
                builder.numEltPerGRF<Type_UB>();
       dstreg = inst->getDst()->getLinearizedStart() /
@@ -2269,7 +2262,8 @@ void SaveRestoreInfo::update(G4_INST *inst, int32_t memOffset,
               mapIt.second.second.offset == offsetForReg) {
             saveRestoreMap.erase(mapIt.first);
 
-            const char *offstr = RegOrMem::MemAbs ? "abs" : "off befp";
+            const char* offstr =
+                (mapIt.second.first == RegOrMem::MemAbs) ? "abs" : "off befp";
             (void)offstr;
             DEBUG_VERBOSE("Restored r" << reg << ".0 (8):d from mem offset "
                                        << offsetForReg << " bytes (" << offstr
