@@ -58,9 +58,9 @@ class VISAKernelImpl : public VISAFunction {
 public:
   VISAKernelImpl(enum VISA_BUILD_TYPE type, CISA_IR_Builder *cisaBuilder,
                  const char *name, unsigned int funcId)
-      : m_mem(4096), m_CISABuilder(cisaBuilder),
-        m_options(cisaBuilder->getOptions()), m_functionId(funcId),
-        m_fastPathOpndPool() {
+      : m_functionId(funcId), m_CISABuilder(cisaBuilder),
+        m_fastPathOpndPool(), m_options(cisaBuilder->getOptions())
+  {
     mBuildOption = m_CISABuilder->getBuilderOption();
     m_magic_number = COMMON_ISA_MAGIC_NUM;
     m_major_version = m_CISABuilder->getMajorVersion();
@@ -1007,7 +1007,8 @@ public:
       VISA_VectorOpnd *renderTargetIndex, vISA_RT_CONTROLS cntrls,
       VISA_StateOpndHandle *surface, VISA_RawOpnd *r1HeaderOpnd,
       VISA_VectorOpnd *sampleIndex, VISA_VectorOpnd *cPSCounter,
-      uint8_t numMsgSpecificOpnds, VISA_RawOpnd **opndArray) override;
+      uint8_t numMsgSpecificOpnds, VISA_RawOpnd **opndArray
+  ) override;
 
   VISA_BUILDER_API int AppendVISA3dURBWrite(
       VISA_PredOpnd *pred, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
@@ -1031,14 +1032,7 @@ public:
   VISA_BUILDER_API int GetErrorMessage(const char *&errorMsg) const override;
   VISA_BUILDER_API virtual int
   GetGenxDebugInfo(void *&buffer, unsigned int &size) const override;
-  /// GetGenRelocEntryBuffer -- allocate and return a buffer of all
-  /// GenRelocEntry that are created by vISA
-  VISA_BUILDER_API int
-  GetGenRelocEntryBuffer(void *&buffer, unsigned int &byteSize,
-                         unsigned int &numEntries) override;
   /// GetRelocations -- add vISA created relocations into given relocation list
-  /// This get the same information as GetGenRelocEntryBuffer, but in different
-  /// foramt
   VISA_BUILDER_API int GetRelocations(RelocListType &relocs) override;
   VISA_BUILDER_API int GetGTPinBuffer(void *&buffer, unsigned int &size,
                                       unsigned int scratchOffset) override;
@@ -1333,7 +1327,7 @@ private:
   std::vector<std::string> m_string_pool;
   enum VISA_BUILD_TYPE m_type;
 
-  vISA::Mem_Manager m_mem;
+  vISA::Mem_Manager m_mem = 4096;
   std::string m_name;
   std::string m_asmName;
   std::string m_sanitizedName;
@@ -1509,6 +1503,7 @@ public:
   uint32_t getInputCount() const { return m_kernel->m_input_count; }
 
   std::string printKernelHeader(bool printVersion);
+  std::string printDeclSection(bool printAsComment);
 
   std::string printInstruction(const CISA_INST *instruction,
                                const Options *opt) const;
