@@ -236,24 +236,24 @@ private:
   uint32_t getDPASOpsPerChan(Type src1_ty, Type src2_ty, bool isDF);
 
 private:
-  Instruction *m_instruction;
+  Instruction *m_instruction = nullptr;
 
   // track the inst id counters when it reach to this instruction
   const InstIDs m_InstIDs;
 
   const DepSetBuilder &m_DB;
 
-  DEP_TYPE m_dType;
-  DEP_PIPE m_dPipe;
-  DEP_CLASS m_dClass;
+  DEP_TYPE m_dType = DEP_TYPE::NONE;
+  DEP_PIPE m_dPipe = DEP_PIPE::NONE;
+  DEP_CLASS m_dClass = DEP_CLASS::NONE;
   BitSet<> *bits;
   std::vector<size_t> m_bucketList;
   SBID m_sbid;
-  bool m_hasIndirect;
+  bool m_hasIndirect = false;
   // set true if the instruction has access the special registers: CR, CE, SR
   // In this case we cannot be sure which register it is actually affect,
   // will need to sync all pipes
-  bool m_hasSR;
+  bool m_hasSR = false;
   void formatShortReg(std::ostream &os, bool &first, const char *reg_name,
                       size_t reg_num, size_t reg_start, size_t reg_len) const;
 
@@ -533,15 +533,6 @@ private:
   private:
     struct SuppressBlock {
     public:
-      // Maximum number of groups allowed. This dpeneds on platform and the src
-      // index
-      const size_t maxNumGroup;
-      const size_t groupSize;
-
-      // Groups in this block: each group is represented by the first register
-      // of the group
-      std::vector<uint16_t> groups;
-
       // Keep tarck of the src and dst register footprints of all instructions
       // those are in this suppressBlcok. This will be used to set to DepSet
       // when the block is decided to be added into the macro, for avoiding
@@ -550,41 +541,7 @@ private:
       RegRangeListType allDstRange;
 
     public:
-      SuppressBlock(size_t maxNumGroup, size_t groupSize)
-          : maxNumGroup(maxNumGroup), groupSize(groupSize) {}
-
-      bool isFull() const { return groups.size() >= maxNumGroup; }
-
-      bool contains(uint16_t startRegNum) const {
-        return std::find(groups.begin(), groups.end(), startRegNum) !=
-               groups.end();
-      }
-
-      // check if the given register range has partially overlapped with existed
-      // ones return true when there is partially overlapped retur false if
-      // there is completely overlapped or no overlapped
-      bool partialOverlapped(uint16_t startRegNum) const {
-        if (contains(startRegNum))
-          return false;
-        for (auto &group : groups) {
-          // number of registers in a group must be all the same across the
-          // groups in the same block
-          uint16_t diff =
-              startRegNum > group ? startRegNum - group : group - startRegNum;
-          if (diff < groupSize)
-            return true;
-        }
-        return false;
-      }
-
-      size_t size() const { return groups.size(); }
-
-      // add the registers into this blcok,
-      void addRegs(uint16_t startRegNum) {
-        assert(!isFull());
-        assert(!contains(startRegNum));
-        groups.push_back(startRegNum);
-      }
+      SuppressBlock() {}
 
       void addRegRanges(SrcRegRangeType &srcRange,
                         SrcRegRangeType &extraSrcRange,
@@ -596,36 +553,14 @@ private:
       }
 
     }; // SuppressionBlock
-    typedef std::unique_ptr<SuppressBlock> SuppressBlockPtrTy;
 
     // get the max number of suppression groups according to srcIdx and platform
+    [[maybe_unused]]
     size_t getNumberOfSuppresionGroups(uint32_t srcIdx) const;
 
     // check from startIt, find the number of consecutive dpas those can be
-    // grouped in a macro due to srcIdx suppression. Return number of
-    // instructions found
-    size_t formSrcSuppressionBlock(InstListIterator startIt, uint32_t srcIdx);
-
-    // return the candidate SuppressBlock that is found fulfilling read
-    // suppression requirement of given src index, start from the give
-    // instruction. This block is the first candidate block of instructions
-    // register those can be suppressed. Will need to check if the following
-    // instructions having the same registers so that they can actually being
-    // suppressed.
-    // * return nullptr if there is no chance to suppress the given src
-    // * allDstBits, allSrcBits - all used grf bits in the return suppressBlock
-    // * allDstNoLastBits, allSrcNoLastBits - all used grf in the return
-    // suppressBlock except the
-    //   last instruction's
-    // * forceGroupNum - force to use the given value as maximum number of
-    // suppression groups instead of
-    //   getting it from getNumberOfSuppresionGroups
-    SuppressBlockPtrTy getSuppressionBlockCandidate(
-        InstListIterator startIt, uint32_t srcIdx, BitSet<> &allDstBits,
-        BitSet<> &allSrcBits, BitSet<> &allDstNoLastBits,
-        BitSet<> &allSrcNoLastBits, int forceGroupNum = -1) const;
-
-    bool srcIsSuppressCandidate(const Instruction &inst, uint32_t srcIdx) const;
+    // grouped in a macro. Return number of instructions found
+    size_t formSrcSuppressionBlock(InstListIterator startIt);
 
   private:
     const DepSetBuilder &m_dsBuilder;
