@@ -89,7 +89,8 @@ public:
 
 kv_t *kv_create(iga_gen_t gen_platf, const void *bytes, size_t bytes_len,
                 iga_status_t *status, char *errbuf, size_t errbuf_cap,
-                uint32_t swsbMode) {
+                uint32_t swsbMode
+                ) {
   if (errbuf && errbuf_cap > 0)
     *errbuf = 0;
 
@@ -107,7 +108,10 @@ kv_t *kv_create(iga_gen_t gen_platf, const void *bytes, size_t bytes_len,
   KernelViewImpl *kvImpl = nullptr;
   try {
     kvImpl =
-        new (std::nothrow) KernelViewImpl(*model, bytes, bytes_len, static_cast<iga::SWSB_ENCODE_MODE>(swsbMode));
+        new (std::nothrow) KernelViewImpl(
+            *model, bytes, bytes_len,
+            static_cast<iga::SWSB_ENCODE_MODE>(swsbMode));
+
     if (!kvImpl) {
       if (errbuf) {
         iga::copyOutString(errbuf, errbuf_cap, nullptr, "failed to allocate");
@@ -362,7 +366,6 @@ kv_status_t kv_get_send_exdesc_immoff(const kv_t *kv, int32_t pc,
     return KV_DESCRIPTOR_INVALID;
   }
 
-  const auto exDesc = inst->getExtMsgDescriptor();
   const auto desc = inst->getMsgDescriptor();
 
   auto addrType = ((desc.imm >> 29) & 0x3); // Desc[30:29]
@@ -453,7 +456,6 @@ kv_status_t kv_get_message_type(const kv_t *kv, int32_t pc,
     return kv_status_t::KV_NON_SEND_INSTRUCTION;
   }
 
-  auto exDesc = inst->getExtMsgDescriptor();
   auto desc = inst->getMsgDescriptor();
 
   // NOTE: we could probably get the message just from desc
