@@ -123,17 +123,21 @@ struct ZEFuncAttribEntry {
   std::string f_name;             // The function's name
   uint8_t f_hasRTCalls;
   uint8_t f_hasPrintfCalls;
+  uint8_t f_requireAssertBuffer;
+  uint8_t f_requireSyncBuffer;
   uint8_t f_hasIndirectCalls;
 
   ZEFuncAttribEntry(uint8_t isKernel, uint8_t isExternal, uint32_t barrierCount,
                     uint32_t privateMemPerThread, uint32_t spillMemPerThread,
                     std::string funcName, uint8_t hasRTCalls,
-                    uint8_t hasPrintfCalls, uint8_t hasIndirectCalls)
+                    uint8_t hasPrintfCalls, uint8_t requireAssertBuffer,
+                    uint8_t requireSyncBuffer, uint8_t hasIndirectCalls)
       : f_isKernel(isKernel), f_isExternal(isExternal),
         f_BarrierCount(barrierCount),
         f_privateMemPerThread(privateMemPerThread),
         f_spillMemPerThread(spillMemPerThread), f_name(std::move(funcName)),
         f_hasRTCalls(hasRTCalls), f_hasPrintfCalls(hasPrintfCalls),
+        f_requireAssertBuffer(requireAssertBuffer), f_requireSyncBuffer(requireSyncBuffer),
         f_hasIndirectCalls(hasIndirectCalls) {}
 };
 
