@@ -62,6 +62,7 @@ DEF_VISA_OPTION(vISA_DumpUndefUsesFromLiveness, ET_BOOL,
 DEF_VISA_OPTION(vISA_disableInstDebugInfo, ET_BOOL, "-disableInstDebugInfo",
                 UNUSED, false)
 DEF_VISA_OPTION(vISA_analyzeMove, ET_BOOL, "-analyzeMove", UNUSED, false)
+DEF_VISA_OPTION(vISA_maxPTSSOverride, ET_INT32, "-maxPTSSOverride", UNUSED, 0)
 DEF_VISA_OPTION(vISA_skipFDE, ET_BOOL, "-skipFDE", UNUSED, false)
 DEF_VISA_OPTION(vISA_storeCE, ET_BOOL, "-storeCE", UNUSED, false)
 // setting this flag makes VISA emit matching name for variable wrt visaasm file
@@ -367,6 +368,9 @@ DEF_VISA_OPTION(vISA_SpillMemOffset, ET_INT32, "-spilloffset",
                 "USAGE: -spilloffset <offset>\n", 0)
 DEF_VISA_OPTION(vISA_ReservedGRFNum, ET_INT32, "-reservedGRFNum",
                 "USAGE: -reservedGRFNum <regNum>\n", 0)
+DEF_VISA_OPTION(vISA_ReservedFromFrontGRFNum, ET_INT32,
+                "-reservedFromFrontGRFNum",
+                "USAGE: -reservedFromFrontGRFNum <regNum>\n", 0)
 DEF_VISA_OPTION(vISA_TotalGRFNum, ET_INT32, "-TotalGRFNum",
                 "USAGE: -TotalGRFNum <regNum>\n", 0)
 DEF_VISA_OPTION(vISA_MaxRAIterations, ET_INT32, "-maxRAIterations",
@@ -573,6 +577,8 @@ DEF_VISA_OPTION(vISA_EnableDPASBundleConflictReduction, ET_BOOL,
 DEF_VISA_OPTION(vISA_NoDPASMacro, ET_BOOL, "-noDPASMacro", UNUSED, false)
 DEF_VISA_OPTION(vISA_forceDPASMacro, ET_BOOL, "-forceDPASMacro", "DEPRECATED, is a nop", false)
 DEF_VISA_OPTION(vISA_KeepDPASMacroInSchedule, ET_BOOL, "-keepDPASMacroInSchedule", UNUSED, false)
+DEF_VISA_OPTION(vISA_scheduleforDPASMacro, ET_BOOL, "-scheduleforDPASMacro",
+                UNUSED, false)
 DEF_VISA_OPTION(vISA_TrueDepOnly, ET_BOOL, "-trueDepOnly", UNUSED, false)
 DEF_VISA_OPTION(vISA_SplitMov64, ET_INT32, "-SplitMov64",
                 "USAGE: -SplitMov64 (0|1|2)\n", 0)
@@ -593,7 +599,7 @@ DEF_VISA_OPTION(vISA_HybridRAWithSpill, ET_BOOL, "-hybridRAWithSpill", UNUSED,
 DEF_VISA_OPTION(vISA_SelectiveFastRA, ET_BOOL, "-selectiveFastRA", UNUSED,
                 false)
 DEF_VISA_OPTION(vISA_SelectiveRAInstThreshold, ET_INT32, "-selectiveRAInstThreshold",
-                UNUSED, 128*1024)
+                UNUSED, 131072) // 128*1024
 DEF_VISA_OPTION(vISA_SelectiveRAGlobaVarRatioThreshold, ET_CSTR, "-selectiveRAGVRatioThreshold",
                 UNUSED, "0.16")
 DEF_VISA_OPTION(vISA_EnableSwapAccSub, ET_BOOL, "-swapAccSub", UNUSED, true)
@@ -617,6 +623,8 @@ DEF_VISA_OPTION(vISA_GenerateCombinedISAASM, ET_BOOL, "-dumpcombinedcisa",
 DEF_VISA_OPTION(vISA_ISAASMToConsole, ET_BOOL, "-isaasmToConsole",
                 "The option can be used with -dumpcommonisa to make finalizer"
                 "emit isaasm to stdout instead of file and do early exit", false)
+DEF_VISA_OPTION(vISA_AddISAASMDeclarationsToEnd, ET_BOOL, "-isaasmAddDeclarationsAtEnd",
+                "Add a comment with .decl section to the end of isaasm console dump. Used in tests.", false)
 DEF_VISA_OPTION(vISA_DumpIsaVarNames, ET_BOOL, "-dumpisavarnames", UNUSED, true)
 DEF_VISA_OPTION(vISA_UniqueLabels, ET_BOOL, "-uniqueLabel", UNUSED, false)
 DEF_VISA_OPTION(vISA_ShaderDumpRegexFilter, ET_CSTR, "-shaderDumpRegexFilter",
@@ -783,6 +791,11 @@ DEF_VISA_OPTION(vISA_ActiveThreadsOnlyBarrier, ET_BOOL,
                 "This enables the active-only bit in workgroup barriers. "
                 "With this option exited threads are not counted in expected "
                 "arrival count total and will not cause hangs.", false)
+DEF_VISA_OPTION(vISA_SplitBarrierID1, ET_BOOL,
+                "-splitbarrierid1",
+                "This flag switch ID of the split barrier to 1. "
+                "After that change, the workgroupbarrier and splitbarrer"
+                "can work togheter and will not cause hangs.", false)
 DEF_VISA_OPTION(vISA_RestrictSrc1ByteSwizzle, ET_BOOL,
                 "-restrictSrc1ByteSwizzle",
                 "Enable the WA to restrict src1 byte swizzle case", false)
